package cn.newfeifan.mall.module.distri.service.integral;

import cn.newfeifan.mall.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.newfeifan.mall.module.distri.controller.admin.ptprofit.vo.PtProfitSaveReqVO;
import cn.newfeifan.mall.module.distri.controller.admin.ptprofitlog.vo.PtProfitLogSaveReqVO;
import cn.newfeifan.mall.module.distri.dal.dataobject.ptprofit.PtProfitDO;
import cn.newfeifan.mall.module.distri.dal.dataobject.ptprofitlog.PtProfitLogDO;
import cn.newfeifan.mall.module.distri.dal.mysql.ptprofitlog.PtProfitLogMapper;
import cn.newfeifan.mall.module.distri.enums.CaclEnum;
import cn.newfeifan.mall.module.distri.service.ptprofit.PtProfitService;
import cn.newfeifan.mall.module.distri.service.ptprofitlog.PtProfitLogService;
import cn.newfeifan.mall.module.distri.service.sharepath.SharePathService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import org.springframework.validation.annotation.Validated;

import java.time.LocalDateTime;
import java.util.*;

import cn.newfeifan.mall.module.distri.controller.admin.integral.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.integral.IntegralDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.framework.common.util.object.BeanUtils;

import cn.newfeifan.mall.module.distri.dal.mysql.integral.IntegralMapper;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.module.distri.constant.DistriConstants.PT_ID;
import static cn.newfeifan.mall.module.distri.enums.CaclEnum.DIRECT_REFERRAL_QUOTA;
import static cn.newfeifan.mall.module.distri.enums.CaclEnum.MANUAL_RETURN_INTEGRAL;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;

/**
 * 推荐用户积分 Service 实现类
 *
 * @author 非繁人
 */
@Service
@Validated
public class IntegralServiceImpl implements IntegralService {

    @Resource
    private IntegralMapper integralMapper;

    @Resource
    private PtProfitLogMapper ptProfitLogMapper;

    @Resource
    @Lazy
    private SharePathService sharePathService;

    @Resource
    private PtProfitLogService ptProfitLogService;

    @Resource
    private PtProfitService ptProfitService;

    @Override
    public Long createIntegral(IntegralSaveReqVO createReqVO) {
        // 插入
        IntegralDO integral = BeanUtils.toBean(createReqVO, IntegralDO.class);
        integral.setAccumulatedQuota(0L);
        integral.setAncestorQuota(0L);
        integralMapper.insert(integral);
        // 返回
        return integral.getId();
    }

    @Override
    public void updateIntegral(IntegralSaveReqVO updateReqVO) {
        // 校验存在
        validateIntegralExists(updateReqVO.getId());
        // 更新
        IntegralDO updateObj = BeanUtils.toBean(updateReqVO, IntegralDO.class);
        integralMapper.updateById(updateObj);
    }

    @Override
    public void deleteIntegral(Long id) {
        // 校验存在
        validateIntegralExists(id);
        // 删除
        integralMapper.deleteById(id);
    }

    private void validateIntegralExists(Long id) {
        if (integralMapper.selectById(id) == null) {
            throw exception(INTEGRAL_NOT_EXISTS);
        }
    }

    @Override
    public IntegralDO getIntegral(Long id) {
        return integralMapper.selectOne(new LambdaQueryWrapperX<IntegralDO>().eqIfPresent(IntegralDO::getUserId, id));
    }

    @Override
    public PageResult<IntegralDO> getIntegralPage(IntegralPageReqVO pageReqVO) {
        return integralMapper.selectPage(pageReqVO);
    }

    @Override
    public IntegralDO selectByUser(Long descendant) {
        return integralMapper.selectOne(new LambdaQueryWrapper<IntegralDO>().eq(IntegralDO::getUserId, descendant));
    }

    @Override
    public void saveBatch(List<IntegralSaveReqVO> integralSaveReqVOS) {
        List<IntegralDO> integralDOS = BeanUtils.toBean(integralSaveReqVOS, IntegralDO.class);
        integralDOS.forEach(k -> {
            k.setAccumulatedQuota(0L);
            k.setAncestorQuota(0L);
        });
        integralMapper.insertBatch(integralDOS);
    }

    @Override
    public void updateBatch(List<IntegralSaveReqVO> integralSaveReqVOS) {

        integralSaveReqVOS.forEach(k -> {
            // 校验存在
            validateIntegralExists(k.getId());
            // 更新
            IntegralDO updateObj = BeanUtils.toBean(k, IntegralDO.class);
            integralMapper.updateById(updateObj);
        });

    }

    @Override
    public IntegralDO selectUserParent(Long userId) {

        Long parentId = sharePathService.queryParentBySonUserId(userId);
        return integralMapper.selectOne(new LambdaQueryWrapperX<IntegralDO>().eqIfPresent(IntegralDO::getUserId, parentId));
    }

    @Override
    public List<IntegralDO> selectAllUser() {

        return integralMapper.selectList();

    }

    @Override
    public IntegralDO getIntegralByUserId(Long userId) {
        return integralMapper.selectOne(new LambdaQueryWrapper<IntegralDO>()
                .eq(IntegralDO::getUserId, userId)
        );
    }

    @Override
    public void updateIntegralFreezeHighQuota(Long generateUserId, Long userId, CaclEnum caclEnum, Long totalFreezeHighQuota, Long tradeOrderId, String orderNum, LocalDateTime payTime) {
        try{
            IntegralDO integralDO = integralMapper.selectOne(new LambdaQueryWrapper<IntegralDO>().eq(IntegralDO::getUserId, userId));
            integralDO.setFreezeHighQuota(integralDO.getFreezeHighQuota() + totalFreezeHighQuota);
            integralMapper.updateById(integralDO);

            //记录用户的积分变动日志
            PtProfitLogDO ptProfitLogDO = PtProfitLogDO.builder()
                    .freezeHighQuota(totalFreezeHighQuota)
                    .afterFreezeHighQuota(integralDO.getFreezeHighQuota())
                    .userId(userId)
                    .generateUserId(generateUserId)
                    .profitStatus(caclEnum.getType())
                    .orderId(tradeOrderId)
                    .orderNo(orderNum)
                    .build();
            ptProfitLogDO.setCreateTime(payTime);
            ptProfitLogDO.setUpdateTime(payTime);
            ptProfitLogMapper.insert(ptProfitLogDO);
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
    }

    @Override
    public void initializeIntegralHighQuotaTotal() {
        List<IntegralDO> integralDOS = integralMapper.selectList();
        for (IntegralDO integralDO : integralDOS) {
            List<PtProfitLogDO> ptProfitLogDOS = ptProfitLogMapper.selectList(new LambdaQueryWrapper<PtProfitLogDO>()
                    .eq(PtProfitLogDO::getUserId, integralDO.getUserId())
                    .eq(PtProfitLogDO::getProfitStatus, DIRECT_REFERRAL_QUOTA.getType())
            );
            for (PtProfitLogDO ptProfitLogDO : ptProfitLogDOS) {
                integralDO.setHighQuotaTotal(integralDO.getHighQuotaTotal() + ptProfitLogDO.getMaxAvailablePointsAmount());
            }

            integralMapper.updateById(integralDO);
            if(ptProfitLogDOS.size() > 1){
                System.out.println("用户编号:[" + integralDO.getUserId() + "]");
            }
        }
    }

    @Override
    public void updateUserIntegral(Long userId, CaclEnum caclEnum, Long payIntegral, Long tradeOrderId, String orderNum) {
        //退还订单支付过期积分
        IntegralDO integralDO = integralMapper.selectOne(new LambdaQueryWrapper<IntegralDO>().eq(IntegralDO::getUserId, userId));
        integralDO.setCurrentQuota(integralDO.getCurrentQuota() + payIntegral);

        integralMapper.updateById(integralDO);

        ptProfitLogService.addMessage(userId,userId,caclEnum,payIntegral,integralDO.getCurrentQuota(),0L,0L,null,tradeOrderId,orderNum);
    }

    @Override
    public void calcIntegral() {
        List<PtProfitLogDO> ptProfitLogDOS = ptProfitLogService.calcIntegral();
        for (PtProfitLogDO ptProfitLogDO : ptProfitLogDOS) {
            IntegralDO integralDO = integralMapper.selectOne(new LambdaQueryWrapper<IntegralDO>().eq(IntegralDO::getUserId, ptProfitLogDO.getUserId()));
            //加积分余额
            integralDO.setCurrentQuota(integralDO.getCurrentQuota() + ptProfitLogDO.getAccumulatedQuotaAmount());
            //加累计积分余额
            integralDO.setAccumulatedQuota(integralDO.getAccumulatedQuota() + ptProfitLogDO.getAccumulatedQuotaAmount());
            //减可用峰值
            integralDO.setHighQuota(integralDO.getHighQuota() - ptProfitLogDO.getAccumulatedQuotaAmount());

            integralMapper.updateById(integralDO);

            PtProfitDO ptProfit = ptProfitService.getPtProfit();
            IntegralDO ptIntegralDO = selectByUser(PT_ID);

            //平台用户扣除积分
            ptProfit.setPtGrossAdd(ptProfit.getPtGrossAdd() - ptProfitLogDO.getAccumulatedQuotaAmount());
            ptIntegralDO.setCurrentQuota(ptIntegralDO.getCurrentQuota() - ptProfitLogDO.getAccumulatedQuotaAmount());
            // 平台扣除积分
            ptProfit.setPtTotalAdd(ptProfit.getPtTotalAdd() - ptProfitLogDO.getAccumulatedQuotaAmount());

            integralMapper.updateById(ptIntegralDO);
            ptProfitService.updatePtProfit(PtProfitSaveReqVO.builder().id(ptProfit.getId())
                    .ptAdd(ptProfit.getPtAdd())
                    .ptGrossAdd(ptProfit.getPtGrossAdd())
                    .ptTotalAdd(ptProfit.getPtTotalAdd()).build());

            //这条记录是用户看的
            PtProfitLogSaveReqVO ProfitLog = PtProfitLogSaveReqVO.builder()
                    .amount(ptProfitLogDO.getAccumulatedQuotaAmount())
                    .afterAmount(integralDO.getCurrentQuota())
                    .userId(ptProfitLogDO.getUserId())
                    .generateUserId(ptProfitLogDO.getGenerateUserId())
                    .profitStatus(MANUAL_RETURN_INTEGRAL.getType())
                    .maxAvailablePointsAmount(-ptProfitLogDO.getAccumulatedQuotaAmount())
                    .afterMaxAvailablePointsAmount(integralDO.getHighQuota())
                    .build();
            ptProfitLogMapper.insert(BeanUtils.toBean(ProfitLog, PtProfitLogDO.class));

            //这条是pt看的
            PtProfitLogSaveReqVO ptProfitLog = PtProfitLogSaveReqVO.builder()
                    .amount(ptProfitLogDO.getAccumulatedQuotaAmount())
                    .afterAmount(ptProfit.getPtGrossAdd())
                    .generateUserId(ptProfitLogDO.getGenerateUserId())
                    .profitStatus(MANUAL_RETURN_INTEGRAL.getType())
                    .build();
            ptProfitLogMapper.insert(BeanUtils.toBean(ptProfitLog, PtProfitLogDO.class));
        }
    }

}
